#include "layoutbrowser.h"
#include "ui_layoutbrowser.h"

#include <QtMath>
#include <QGuiApplication>
#include <QFileSystemModel>
#include <QDebug>
#include <QPainter>
#include <QPixmapCache>
#include <QItemSelectionModel>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QPushButton>

#include "flowlayout.h"

LayoutBrowser::LayoutBrowser(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LayoutBrowser),
    m_orientation(0)
{
    ui->setupUi(this);
    ui->tags_widget->setLayout(new FlowLayout(ui->tags_widget));

    QPushButton *but = new QPushButton(tr("All"), ui->tags_widget);
    but->setStyleSheet("color:palette(text)");
    but->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
    but->setCheckable(true);
    but->setChecked(true);
    but->setEnabled(false);
    connect(but, &QPushButton::toggled, this, &LayoutBrowser::onTagsChanged);
    ui->tags_widget->layout()->addWidget(but);
    m_tagButtons << but;

    Q_FOREACH(const QString &tag, ComicLayout::tags().keys()) {
        QPushButton *but = new QPushButton(tr(tag.toUtf8()), ui->tags_widget);
        but->setStyleSheet("color:palette(text)");
        but->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
        but->setCheckable(true);
        connect(but, &QPushButton::toggled, this, &LayoutBrowser::onTagsChanged);
        ui->tags_widget->layout()->addWidget(but);
        m_tagButtons << but;
    }

    LayoutsTableModel *model = new LayoutsTableModel(ComicLayout::layouts(), ui->tableView);
    model->setLayoutBrowser(this);

    LayoutDelegate *delegate = new LayoutDelegate(ui->tableView);
    delegate->setLayoutBrowser(this);

    ui->tableView->setItemDelegate(delegate);
    ui->tableView->setModel(model);
    QModelIndex index;

    connect(this, SIGNAL(orientationChanged(int)), ui->tableView, SLOT(doItemsLayout()));
    connect(this, SIGNAL(resized()), model, SIGNAL(modelReset()));
    connect(model, SIGNAL(filesChanged(QStringList)), model, SIGNAL(modelReset()));
    connect(ui->tableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SIGNAL(selectionChanged()));
    connect(model, SIGNAL(modelReset()), this, SIGNAL(selectionChanged()));
}

LayoutBrowser::~LayoutBrowser()
{
    delete ui;
}

QString LayoutBrowser::selectedLayout() const
{

    LayoutsTableModel *model = (LayoutsTableModel*)ui->tableView->model();
    int idx = ui->tableView->currentIndex().row()*model->itemsPerRow()+ui->tableView->currentIndex().column();
    if (idx >= 0) {
        return model->files().at(idx);
    } else {
        return QString();
    }
}

int LayoutBrowser::orientation() const
{
    return m_orientation;
}

void LayoutBrowser::setOrientation(int orientation)
{
    if (m_orientation == orientation)
        return;

    m_orientation = orientation;
    emit orientationChanged(orientation);
}

void LayoutBrowser::onTagsChanged(bool checked)
{
    if (!checked) { return; }
    QPushButton *sendr = qobject_cast<QPushButton*>(sender());
    sendr->setDisabled(true);
    Q_FOREACH(QPushButton *button, m_tagButtons) {
        if (sendr != button) {
            button->setChecked(false);
            button->setEnabled(true);
        }
    }
    if (sendr == m_tagButtons[0]) {
        qobject_cast<LayoutsTableModel*>(ui->tableView->model())->setFiles(ComicLayout::layouts());

    } else {
        qobject_cast<LayoutsTableModel*>(ui->tableView->model())->setFiles(ComicLayout::tags()[sendr->text()]);
    }
}


LayoutsTableModel::LayoutsTableModel(QStringList files, QObject *parent) :
    QAbstractItemModel(parent),
    m_files(files)
{
}

int LayoutsTableModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    int rows = qCeil((float)m_files.length()/itemsPerRow());
    return rows;
}

int LayoutsTableModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return itemsPerRow();
}

QVariant LayoutsTableModel::data(const QModelIndex &index, int role) const
{
    int id = index.row()*itemsPerRow()+index.column();
    if (id >= m_files.length()) {
        return QVariant();
    }

    switch (role) {
    case Qt::DisplayRole: {
        return m_files.at(id);
    }
    default: {
        return QVariant();
    }
    }
}

int LayoutsTableModel::itemsPerRow() const
{
    QTableView *tableView = qobject_cast<QTableView*>(QObject::parent());
    float width = tableView->width();
    int cols = qMax(qFloor(width/128), 1);
    int rows = qCeil((float)m_files.size()/cols);
    if ((rows*128) > tableView->height()) {
        // has vertical scrollbar
        width = tableView->width() - tableView->style()->pixelMetric(QStyle::PM_ScrollBarExtent);
        cols = qMax(qFloor(width/128), 1);
    }
    return cols;
}

void LayoutsTableModel::setLayoutBrowser(LayoutBrowser *layoutBrowser)
{
    m_layoutBrowser = layoutBrowser;
}

QStringList LayoutsTableModel::files() const
{
    return m_files;
}

void LayoutDelegate::setLayoutBrowser(LayoutBrowser *layoutBrowser)
{
    m_layoutBrowser = layoutBrowser;
}

void LayoutsTableModel::setItemsPerRow(int itemsPerRow)
{
    if (m_itemsPerRow == itemsPerRow)
        return;

    m_itemsPerRow = itemsPerRow;
    emit itemsPerRowChanged(itemsPerRow);
}

void LayoutsTableModel::setFiles(QStringList files)
{
    if (m_files == files)
        return;

    m_files = files;
    emit filesChanged(files);
}


QModelIndex LayoutsTableModel::index(int row, int column, const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    int id = row*itemsPerRow()+column;
    if (id >= m_files.length()) {
        return QModelIndex();
    }

    return createIndex(row, column);
}

QModelIndex LayoutsTableModel::parent(const QModelIndex &child) const
{
    Q_UNUSED(child)
    return QModelIndex();
}


LayoutDelegate::LayoutDelegate(QObject *parent) :
    QAbstractItemDelegate(parent)
{

}

void LayoutDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QString layoutName = index.data().toString();

    if (m_layoutBrowser->orientation() == 0) {
        painter->drawPixmap(option.rect.x()+22, option.rect.y()+4, ComicLayout::preview(layoutName, 84, 120));
    } else {
        painter->drawPixmap(option.rect.x()+4, option.rect.y()+22, ComicLayout::preview(layoutName, 120, 84));
    }
}

QSize LayoutDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(option); Q_UNUSED(index)
    return QSize(128,128);
}


void LayoutBrowser::resizeEvent(QResizeEvent *evt)
{
    QWidget::resizeEvent(evt);
    emit resized();
}
