#include "krapagegenerator.h"
#include "shapelayer.h"

#include <quazip.h>
#include <quazipfile.h>
#include <quazipnewinfo.h>

#include <QDebug>
#include <QUuid>
#include <QPixmap>
#include <QJsonArray>
#include <QJsonObject>
#include <QFileInfo>

#include <comicutils.h>
#include <comicproject.h>
#include <comicpage.h>
#include <comiclayout.h>

struct KraPageGenerator::Private {
    Private(KraPageGenerator *p) :
        p(p)
    {}

    void generate(const QString &filename);
    void generateBegin(const QString &title);
    void generateCropMarkings(const QString &title);
    void generateLayout(const QString &title);
    void generateEnd(const QString &title);

    void groupLayerBegin(const QString &name);
    void groupLayerEnd();
    QString cloneLayer(const QString &name, const QString &cloneUuid);

    QString maindoc_xml;
    QSharedPointer<QuaZip> archive;
    KraPageGenerator *p;
};

KraPageGenerator::KraPageGenerator() :
    PageGenerator(),
    m_private(new Private(this))
{

}

void KraPageGenerator::generate(const QString &filename)
{
    m_private->generate(filename);
}

void KraPageGenerator::Private::generate(const QString &filename)
{
    archive = archive.create();
    archive->setZipName(filename);
    archive->open( QuaZip::mdCreate);

    QString title = QFileInfo(filename).baseName();

    generateBegin(title);
    generateCropMarkings(title);
    generateLayout(title);
    generateEnd(title);

    archive->close();
}

void KraPageGenerator::Private::generateBegin(const QString &title)
{
    QSharedPointer<ComicPage> page = p->comicPage();
    maindoc_xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
                    "<!DOCTYPE DOC PUBLIC '-//KDE//DTD krita 2.0//EN' 'http://www.calligra.org/DTD/krita-2.0.dtd'>\n"
                    "<DOC xmlns=\"http://www.calligra.org/DTD/krita\" editor=\"Krita\" syntaxVersion=\"2\">\n";
    maindoc_xml += QString(" <IMAGE description=\"\" y-res=\"%1\" colorspacename=\"GRAYA\" x-res=\"%1\" mime=\"application/x-kra\" "
                           "height=\"%3\" name=\"%4\" width=\"%2\" profile=\"gray built-in\">\n").arg(page->resolution()).arg(ComicUtils::mmToPx(page->manuscriptSize().width(), page->resolution())).arg(ComicUtils::mmToPx(page->manuscriptSize().height(), page->resolution())).arg(title);
    maindoc_xml += "  <layers>\n";
}

void KraPageGenerator::Private::generateCropMarkings(const QString &title)
{
    QSharedPointer<ComicPage> page = p->comicPage();
    if ((page->manuscriptSize().width() <= page->bindingSize().width()) || (page->manuscriptSize().height() <= page->bindingSize().height())) {
        return;
    }

    float manuscriptWidthPt = ComicUtils::mmToPt(page->manuscriptSize().width());
    float manuscriptHeightPt = ComicUtils::mmToPt(page->manuscriptSize().height());
    float bindingWidthPt = ComicUtils::mmToPt(page->bindingSize().width());
    float bindingHeightPt = ComicUtils::mmToPt(page->bindingSize().height());
    float bleedWidthPt = ComicUtils::mmToPt(page->bleedWidth());
    float cropLineStrength = 0.48;


    QRectF finishedRect(manuscriptWidthPt/2-bindingWidthPt/2,manuscriptHeightPt/2-bindingHeightPt/2,bindingWidthPt,bindingHeightPt);

    ShapeLayer finishedSizeLayer(tr("Finished Size"), p->comicPage()->layout()->lineStrength());
    finishedSizeLayer.drawRect(finishedRect, 0);
    finishedSizeLayer.writeFiles(archive.data(), title);
    maindoc_xml += finishedSizeLayer.maindoc_xml();

    ShapeLayer layer(tr("Cutting Marks"), p->comicPage()->layout()->lineStrength());

    // Bleed Size
    QRectF bleedRect;
    bleedRect.setWidth(bindingWidthPt+bleedWidthPt*2);
    bleedRect.setHeight(bindingHeightPt+bleedWidthPt*2);
    bleedRect.translate(manuscriptWidthPt/2-bleedRect.width()/2, manuscriptHeightPt/2-bleedRect.height()/2);
    layer.drawRect(bleedRect, 0);

    float lineLength = qMin(qMin(bleedRect.x(), bleedRect.y()), 28.0);

    // Horizontal markings
    layer.drawRect(QRectF(bleedRect.right(),finishedRect.y()-cropLineStrength/2, lineLength, cropLineStrength),1);
    layer.drawRect(QRectF(bleedRect.right(),finishedRect.center().y()-cropLineStrength/2, lineLength, cropLineStrength),1);
    layer.drawRect(QRectF(bleedRect.right(),finishedRect.bottom()-cropLineStrength/2, lineLength, cropLineStrength),1);

    layer.drawRect(QRectF(bleedRect.x()- lineLength,finishedRect.y()-cropLineStrength/2, lineLength, cropLineStrength),1);
    layer.drawRect(QRectF(bleedRect.x()- lineLength,finishedRect.center().y()-cropLineStrength/2, lineLength, cropLineStrength),1);
    layer.drawRect(QRectF(bleedRect.x()- lineLength,finishedRect.bottom()-cropLineStrength/2, lineLength, cropLineStrength),1);

    layer.drawRect(QRectF(finishedRect.center().x()-lineLength, bleedRect.y()-lineLength/2-cropLineStrength/2, lineLength*2, cropLineStrength), 1);
    layer.drawRect(QRectF(finishedRect.center().x()-lineLength, bleedRect.bottom()+lineLength/2-cropLineStrength/2, lineLength*2, cropLineStrength), 1);

    // Vertical markings
    layer.drawRect(QRectF(finishedRect.x()-cropLineStrength/2, bleedRect.y()-lineLength, cropLineStrength, lineLength), 1);
    layer.drawRect(QRectF(finishedRect.center().x()-cropLineStrength/2, bleedRect.y()-lineLength, cropLineStrength, lineLength), 1);
    layer.drawRect(QRectF(finishedRect.right()-cropLineStrength/2, bleedRect.y()-lineLength, cropLineStrength, lineLength), 1);

    layer.drawRect(QRectF(finishedRect.x()-cropLineStrength/2, bleedRect.bottom(), cropLineStrength, lineLength), 1);
    layer.drawRect(QRectF(finishedRect.center().x()-cropLineStrength/2, bleedRect.bottom(), cropLineStrength, lineLength), 1);
    layer.drawRect(QRectF(finishedRect.right()-cropLineStrength/2, bleedRect.bottom(), cropLineStrength, lineLength), 1);

    layer.drawRect(QRectF(bleedRect.x()-lineLength/2-cropLineStrength/2, finishedRect.center().y()-lineLength, cropLineStrength, lineLength*2), 1);
    layer.drawRect(QRectF(bleedRect.right()+lineLength/2-cropLineStrength/2, finishedRect.center().y()-lineLength, cropLineStrength, lineLength*2), 1);



    layer.writeFiles(archive.data(), title);
    maindoc_xml += layer.maindoc_xml();
}

void KraPageGenerator::Private::generateLayout(const QString &title)
{
    QSharedPointer<ComicPage> page = p->comicPage();

    QuaZipFile outFile(archive.data());
    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QStringLiteral("preview.png")));
    if (page->manuscriptSize().height() > page->manuscriptSize().width()) {
        ComicLayout::preview(page->layout()->filename(), 180, 256).save(&outFile, "PNG");
    } else {
        ComicLayout::preview(page->layout()->filename(), 256, 180).save(&outFile, "PNG");
    }
    outFile.close();

    int frameCounter = 1;

    int margins = ComicUtils::mmToPt(p->comicPage()->layout()->margins());

    float manuscriptWidthPt = ComicUtils::mmToPt(page->manuscriptSize().width());
    float manuscriptHeightPt = ComicUtils::mmToPt(page->manuscriptSize().height());
    float bindingWidthPt = ComicUtils::mmToPt(page->bindingSize().width());
    float bindingHeightPt = ComicUtils::mmToPt(page->bindingSize().height());

    //QRect effectiveRect = m_finishedRect.adjusted(margins, margins, -margins, -margins);

    QRect finishedRect(manuscriptWidthPt/2-bindingWidthPt/2,manuscriptHeightPt/2-bindingHeightPt/2,bindingWidthPt,bindingHeightPt);
    float lineStrength = ComicUtils::mmToPt(p->comicPage()->layout()->lineStrength());
    finishedRect.adjust(lineStrength/2, lineStrength/2, -lineStrength/2, -lineStrength/2);

    Q_FOREACH (const QJsonValue &v, page->layout()->layout()) {
        QJsonObject obj = v.toObject();
        QString type(obj.value("type").toString());

        //qDebug() << obj.value("type").toString();

        if (type == "rect") {
            //qDebug() << "Creating rect";
            QJsonObject data = obj.value("data").toObject();
            QRectF rect(data.value("x").toDouble(), data.value("y").toDouble(), data.value("width").toDouble(), data.value("height").toDouble());

            groupLayerBegin(QString(tr("Frame %1")).arg(frameCounter));

            ShapeLayer boxLayer(tr("Box"), p->comicPage()->layout()->lineStrength());
            boxLayer.drawScaledBox(finishedRect.adjusted(margins, margins, -margins, -margins), rect);
            boxLayer.writeFiles(archive.data(), title);

            cloneLayer(tr("Frame"), boxLayer.uuid());

            maindoc_xml += boxLayer.maindoc_xml();

            groupLayerEnd();
        }
        frameCounter++;
    }
}

void KraPageGenerator::Private::generateEnd(const QString &title)
{
    QSharedPointer<ComicPage> page = p->comicPage();
    ShapeLayer layer2(tr("Paper"), p->comicPage()->layout()->lineStrength());
    layer2.drawRect(QRectF(0, 0, ComicUtils::mmToPt(page->manuscriptSize().width()), ComicUtils::mmToPt(page->manuscriptSize().height())),2);
    layer2.writeFiles(archive.data(), title);
    maindoc_xml += layer2.maindoc_xml();

    QuaZipFile outFile(archive.data());

    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QStringLiteral("mimetype")));
    outFile.write("application/x-krita");
    outFile.close();

    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QStringLiteral("documentinfo.xml")));
    outFile.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
                  "<!DOCTYPE document-info PUBLIC '-//KDE//DTD document-info 1.1//EN' 'http://www.calligra.org/DTD/document-info-1.1.dtd'>"
                  "<document-info xmlns=\"http://www.calligra.org/DTD/document-info\">"
                  "</document-info>");
    outFile.close();

    maindoc_xml += "\n  </layers>\n </IMAGE>\n</DOC>";
    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QStringLiteral("maindoc.xml")));
    outFile.write(maindoc_xml.toUtf8());
    outFile.close();
}

void KraPageGenerator::Private::groupLayerBegin(const QString &name)
{
    maindoc_xml += QString("<layer uuid=\"%1\" visible=\"1\" x=\"0\" nodetype=\"grouplayer\" channelflags=\"\" name=\"%2\" passthrough=\"0\" colorlabel=\"0\" locked=\"0\" y=\"0\" collapsed=\"1\" opacity=\"255\" compositeop=\"normal\">\n"
                     "  <layers>").arg(QUuid::createUuid().toString()).arg(name);
}

void KraPageGenerator::Private::groupLayerEnd()
{
    maindoc_xml += "</layers>\n</layer>\n";
}

QString KraPageGenerator::Private::cloneLayer(const QString &name, const QString &cloneUuid)
{
    QString uuid = QUuid::createUuid().toString();
    maindoc_xml += QString("<layer collapsed=\"0\" locked=\"1\" uuid=\"%1\" clonetype=\"0\" compositeop=\"multiply\" channelflags=\"\" visible=\"1\" nodetype=\"clonelayer\" clonefromuuid=\"%2\" name=\"%3\" colorlabel=\"0\" opacity=\"255\" x=\"0\" y=\"0\" selected=\"true\"/>").arg(uuid).arg(cloneUuid).arg(name);
    return uuid;
}
