#include "comiclayout.h"

#include <QSettings>
#include <QMetaEnum>
#include <QDebug>
#include <QPainter>
#include <QPixmapCache>
#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonParseError>
#include <QDir>

struct ComicLayout::Private {
    Private() :
        orientation(Portrait),
        margins(15),
        lineStrength(1)
    {}
    void save(QSharedPointer<QSettings> conf)
    {
        conf->setValue("filename", filename);
        conf->setValue("orientation", QMetaEnum::fromType<Orientation>().key(orientation));
        conf->setValue("margins", margins);
        conf->setValue("lineStrength", lineStrength);
    }
    void load(QSharedPointer<QSettings> conf)
    {
        filename = conf->value("filename").toString();
        orientation = (Orientation)QMetaEnum::fromType<Orientation>().keyToValue(conf->value("orientation").toString().toLocal8Bit().constData());
        margins = conf->value("margins").toFloat();
        lineStrength = conf->value("lineStrength").toFloat();
    }

    QString filename;
    Orientation orientation;
    QJsonArray layout;
    float margins;
    float lineStrength;

    static QStringList layouts;
    static QMap<QString,QStringList> tags;
};

QStringList ComicLayout::Private::layouts;
QMap<QString,QStringList> ComicLayout::Private::tags;


ComicLayout::ComicLayout(QObject *parent) :
    QObject(parent),
    m_private(new Private)
{
    setFilename(QString());
    setOrientation(Portrait);
}

ComicLayout::ComicLayout(QString layoutFilename, ComicLayout::Orientation orientation, QObject *parent) :
    QObject(parent),
    m_private(new Private)
{
    setFilename(layoutFilename);
    setOrientation(orientation);
}

ComicLayout::ComicLayout(const ComicLayout &other) :
    QObject(other.parent()),
    m_private(new Private(*other.m_private.data()))
{

}

ComicLayout &ComicLayout::operator=(const ComicLayout &other)
{
    QObject(other.parent());
    m_private = QSharedPointer<Private>(new Private(*other.m_private.data()));
    return *this;
}

ComicLayout::~ComicLayout()
{
    //qDebug() << "~ComicLayout()";
}

void ComicLayout::save(QSharedPointer<QSettings> conf)
{
    m_private->save(conf);
}

QSharedPointer<ComicLayout> ComicLayout::load(QSharedPointer<QSettings> conf)
{
    QSharedPointer<ComicLayout> ret(new ComicLayout);
    ret->m_private->load(conf);
    ret->load();
    return ret;
}

bool ComicLayout::load(QByteArray json)
{
    QJsonParseError error;
    QJsonDocument doc = QJsonDocument::fromJson(json, &error);
    if (error.error != error.NoError) {
        qDebug() << error.errorString();
        qDebug() << error.offset;
        return false;
    }
    m_private->layout = doc.object().value("layout").toArray();

    return true;
}

bool ComicLayout::load()
{

    QFile sourceFile(m_private->filename);
    bool ret = false;
    if (sourceFile.exists()) {
        sourceFile.open(QIODevice::ReadOnly);
        ret = load(sourceFile.readAll());
    } else {
        qDebug() << "Warning: layout file" << m_private->filename << "not found.";
    }
    return ret;
}

QJsonArray ComicLayout::layout() const
{
    return m_private->layout;
}

QPixmap ComicLayout::preview() const
{
    switch (orientation()) {
    default:
    case Portrait:
        return preview(m_private->filename, 84, 120);
        break;
    case Landscape:
        return preview(m_private->filename, 120, 84);
        break;
    }
}

QPixmap ComicLayout::preview(QJsonArray layout, int width, int height)
{
    Q_UNUSED(layout); Q_UNUSED(width); Q_UNUSED(height);
    //QPixmapCache::find
    return QPixmap();
}

QPixmap ComicLayout::preview(QString filename, int width, int height)
{
    int margin = 20;
    width *= 4;
    height *= 4;
    QString key = QString("%1.%2.%3").arg(filename).arg(width).arg(height);
    QPixmap pixmap;
    if (!QPixmapCache::find(key, &pixmap)) {
        // generate preview
        QFile sourceFile(filename);
        sourceFile.open(QIODevice::ReadOnly);
        QJsonParseError error;
        QJsonDocument doc = QJsonDocument::fromJson(sourceFile.readAll(), &error);
        sourceFile.close();
        if (error.error != error.NoError) {
            qDebug() << error.errorString();
            qDebug() << error.offset;
            return QPixmap();
        }
        QImage image(width, height, QImage::Format_ARGB32);
        QPainter painter(&image);
        painter.fillRect(image.rect(), Qt::white);
        painter.setFont(QFont("sans", 28));

        int counter = 1;

        Q_FOREACH (const QJsonValue &v, doc.object().value("layout").toArray()) {
            QJsonObject obj = v.toObject();
            QString type(obj.value("type").toString());
            if (type == "rect") {
                QJsonObject data = obj.value("data").toObject();
                QRect rect(data.value("x").toDouble()*(width-margin*2)+margin+2,
                           data.value("y").toDouble()*(height-margin*2)+margin+2,
                           data.value("width").toDouble()*(width-margin*2)-4,
                           data.value("height").toDouble()*(height-margin*2)-4);

                painter.drawRect(rect);

                painter.drawText(rect, QString::number(counter), QTextOption(Qt::AlignCenter));
            }
            counter++;
        }

        pixmap.convertFromImage(image.scaled(width/4, height/4, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        QPixmapCache::insert(key, pixmap);
    }
    return pixmap;
}

void ComicLayout::addLayoutDirectory(const QString &directory)
{
    QStringList newFiles = QDir(directory).entryList(QStringList() << "*.json");
    if (newFiles.isEmpty()) {
        return;
    }

    Q_FOREACH(const QString &file, newFiles) {
        QString filename = QString("%1/%2").arg(directory).arg(file);
        QFile inFile(filename);
        inFile.open(QIODevice::ReadOnly);
        QJsonDocument doc = QJsonDocument::fromJson(inFile.readAll());
        inFile.close();
        int num_frames = doc.object().value("layout").toArray().size();
        QString key = QString(tr("%n Frames", "", num_frames));
        Private::tags[key] << filename;
    }

    Private::layouts.clear();
    for (int i=0; i<Private::tags.size(); i++) {
        QString key = QString(tr("%n Frames", "", i));
        if (Private::tags.contains(key)) {
            Private::layouts << Private::tags.value(key);
        }
    }

    Q_FOREACH(const QString &file, newFiles) {
        QString filename = QString("%1/%2").arg(directory).arg(file);
        QFile inFile(filename);
        inFile.open(QIODevice::ReadOnly);
        QJsonDocument doc = QJsonDocument::fromJson(inFile.readAll());
        inFile.close();
        QJsonArray tags = doc.object().value("tags").toArray();
        Q_FOREACH(const QJsonValue &val, tags) {
            QString tag = tr(val.toString().toUtf8());
            Private::tags[tag] << filename;
        }
    }
}

const QStringList &ComicLayout::layouts()
{
    return Private::layouts;
}

const QMap<QString, QStringList> ComicLayout::tags()
{
    return Private::tags;
}

QString ComicLayout::filename() const
{
    return m_private->filename;
}

ComicLayout::Orientation ComicLayout::orientation() const
{
    return m_private->orientation;
}

float ComicLayout::margins() const
{
    return m_private->margins;
}

float ComicLayout::lineStrength() const
{
    return m_private->lineStrength;
}

void ComicLayout::setFilename(QString layoutFilename)
{
    if (m_private->filename == layoutFilename)
        return;

    m_private->filename = layoutFilename;
    load();
    emit filenameChanged(layoutFilename);
}

void ComicLayout::setOrientation(ComicLayout::Orientation orientation)
{
    if (m_private->orientation == orientation)
        return;

    m_private->orientation = orientation;
    emit orientationChanged(orientation);
}

void ComicLayout::setMargins(float margins)
{
    if (m_private->margins == margins)
        return;

    m_private->margins = margins;
    emit marginsChanged(margins);
}

void ComicLayout::setMargins(double margins)
{
    setMargins((float)margins);
}

void ComicLayout::setLineStrength(float lineStrength)
{
    if (m_private->lineStrength == lineStrength)
        return;

    m_private->lineStrength = lineStrength;
    emit lineStrengthChanged(lineStrength);
}

void ComicLayout::setLineStrength(double lineStrength)
{
    setLineStrength((float)lineStrength);
}

