#include "comicpage.h"
#include "comicproject.h"
#include "comiclayout.h"

#include <QSettings>
#include <QDebug>

struct ComicPage::Private {
    explicit Private() :
        bindingSize(QSize(182, 257)),
        manuscriptSize(QSize(210, 297)),
        bleedWidth(5),
        resolution(300)
    {}
    Private(const Private &other)
    {
        layout = QSharedPointer<ComicLayout>(new ComicLayout(*other.layout));
        bindingSize = other.bindingSize;
        manuscriptSize = other.manuscriptSize;
        bleedWidth = other.bleedWidth;
        resolution = other.resolution;
    }
    void save(QSharedPointer<QSettings> conf)
    {
        conf->beginGroup("Layout");
        layout->save(conf);
        conf->endGroup();
        conf->setValue("bindingSize", bindingSize);
        conf->setValue("manuscriptSize", manuscriptSize);
        conf->setValue("bleedWidth", bleedWidth);
        conf->setValue("resolution", resolution);
    }
    void load(QSharedPointer<QSettings> conf)
    {
        conf->beginGroup("Layout");
        layout = ComicLayout::load(conf);
        conf->endGroup();
        bindingSize = conf->value("bindingSize").toSize();
        manuscriptSize = conf->value("manuscriptSize").toSize();
        bleedWidth = conf->value("bleedWidth").toInt();
        resolution = conf->value("resolution").toInt();
    }

    QSharedPointer<ComicLayout> layout;
    QSize bindingSize;
    QSize manuscriptSize;
    int bleedWidth;
    int resolution;
};

ComicPage::ComicPage(QObject *parent) :
    QObject(parent),
    m_private(new Private)
{

}

ComicPage::ComicPage(QString layoutFilename, ComicLayout::Orientation orientation, QObject *parent) :
    QObject(parent),
    m_private(new Private)
{
    setLayout(QSharedPointer<ComicLayout>(new ComicLayout(layoutFilename, orientation)));
}

ComicPage::ComicPage(const ComicPage &other) :
    QObject(other.parent()),
    m_private(new Private(*other.m_private.data()))
{

}

ComicPage &ComicPage::operator=(const ComicPage &other)
{
    QObject(other.parent());
    m_private = QSharedPointer<Private>(new Private(*other.m_private.data()));
    return *this;
}

ComicPage::~ComicPage()
{
    //qDebug() << "~ComicPage()";
}

void ComicPage::save(QSharedPointer<QSettings> conf)
{
    m_private->save(conf);
}

QSharedPointer<ComicPage> ComicPage::load(QSharedPointer<QSettings> conf)
{
    QSharedPointer<ComicPage> ret(new ComicPage);
    ret->m_private->load(conf);
    return ret;
}

int ComicPage::index() const
{
    ComicProject *proj = qobject_cast<ComicProject*>(parent());
    if (proj) {
        return 0;//proj->pages().indexOf(this);
    } else {
        return -1;
    }
}

QSharedPointer<ComicLayout> ComicPage::layout() const
{
    return m_private->layout;
}

QSize ComicPage::bindingSize() const
{
    return m_private->bindingSize;
}

QSize ComicPage::manuscriptSize() const
{
    return m_private->manuscriptSize;
}

int ComicPage::bleedWidth() const
{
    return m_private->bleedWidth;
}

int ComicPage::resolution() const
{
    return m_private->resolution;
}

void ComicPage::setLayout(QSharedPointer<ComicLayout> layout)
{
    if (m_private->layout == layout)
        return;

    m_private->layout = layout;
    emit layoutChanged(layout);
}

void ComicPage::setBindingSize(QSize bindingSize)
{
    if (m_private->bindingSize == bindingSize)
        return;

    m_private->bindingSize = bindingSize;
    emit bindingSizeChanged(bindingSize);
}

void ComicPage::setManuscriptSize(QSize manuscriptSize)
{
    if (m_private->manuscriptSize == manuscriptSize)
        return;

    m_private->manuscriptSize = manuscriptSize;
    emit manuscriptSizeChanged(manuscriptSize);
}

void ComicPage::setBleedWidth(int bleedWidth)
{
    if (m_private->bleedWidth == bleedWidth)
        return;

    m_private->bleedWidth = bleedWidth;
    emit bleedWidthChanged(bleedWidth);
}

void ComicPage::setResolution(int resolution)
{
    if (m_private->resolution == resolution)
        return;

    m_private->resolution = resolution;
    emit resolutionChanged(resolution);
}

