#include "comicproject.h"
#include "comicpage.h"

#include <QSettings>
#include <QDebug>
#include <comicutils.h>

struct ComicProject::Private {
    explicit Private() {}
    Private(const Private &other) {
        foldername = other.foldername;
        author = other.author;
        title = other.title;
        Q_FOREACH(QSharedPointer<ComicPage> item, other.pages) {
            pages.append(QSharedPointer<ComicPage>(new ComicPage(*item.data())));
        }
    }
    void save(QSharedPointer<QSettings> conf)
    {
        conf->beginGroup("Project");
        conf->setValue("author", author);
        conf->setValue("title", title);
        conf->endGroup();

        conf->beginGroup("Pages");
        QByteArray tmp = QString("%.%1i").arg(ComicUtils::digits(pages.size())).toLocal8Bit();
        for (int i=0; i<pages.size(); i++) {
            conf->beginGroup(QString().sprintf(tmp, i+1));
            pages.at(i)->save(conf);
            conf->endGroup();
        }
        conf->endGroup();
    }
    void load(QSharedPointer<QSettings> conf)
    {
        conf->beginGroup("Project");
        author = conf->value("author").toString();
        title = conf->value("title").toString();
        conf->endGroup();

        conf->beginGroup("Pages");
        QStringList pageList = conf->childGroups();
        QByteArray tmp = QString("%.%1i").arg(ComicUtils::digits(pageList.size())).toLocal8Bit();
        pageList.sort();
        for (int i=0; i<pageList.size(); i++) {
            conf->beginGroup(QString().sprintf(tmp, i+1));
            pages.append(ComicPage::load(conf));
            conf->endGroup();
        }
        conf->endGroup();
    }

    QString foldername;
    QString author;
    QString title;
    QList<QSharedPointer<ComicPage> > pages;
};

ComicProject::ComicProject(QObject *parent) :
    QObject(parent),
    m_private(new Private)
{

}

ComicProject::ComicProject(const ComicProject &other) :
    QObject(other.parent()),
    m_private(new Private(*other.m_private.data()))
{

}

ComicProject &ComicProject::operator=(const ComicProject &other)
{
    QObject(other.parent());
    m_private = QSharedPointer<Private>(new Private(*other.m_private.data()));
    return *this;
}

ComicProject::~ComicProject()
{
    //qDebug() << "~ComicProject()";
}

bool ComicProject::save(QString foldername)
{
    if (foldername.isEmpty()) {
        foldername = m_private->foldername;
    }
    if (foldername.isEmpty()) {
        return false;
    }

    QSharedPointer<QSettings> conf(new QSettings(QString("%1/ComicCreator.ini").arg(m_private->foldername), QSettings::IniFormat));
    conf->clear();
    save(conf);
    return true;
}

void ComicProject::save(QSharedPointer<QSettings> conf)
{
    m_private->save(conf);
}

QSharedPointer<ComicProject> ComicProject::load(QSharedPointer<QSettings> conf)
{
    QSharedPointer<ComicProject> ret(new ComicProject);
    ret->m_private->load(conf);
    return ret;
}

QSharedPointer<ComicProject> ComicProject::load(QString foldername)
{
    QSharedPointer<QSettings> conf(new QSettings(QString("%1/ComicCreator.ini").arg(foldername), QSettings::IniFormat));
    QSharedPointer<ComicProject> ret = load(conf);
    ret->setFoldername(foldername);
    return ret;
}

QSharedPointer<ComicPage> ComicProject::insertPage(QSharedPointer<ComicPage> page, int index)
{
    if (index == -1) {
        index = m_private->pages.size();
    }
    m_private->pages.insert(index, page);
    return page;
}

QSharedPointer<ComicPage> ComicProject::insertPage(ComicPage *page, int index)
{
    return insertPage(QSharedPointer<ComicPage>(page), index);
}

QSharedPointer<ComicPage> ComicProject::insertPage(QString layoutFilename, ComicLayout::Orientation orientation, int index)
{
    return insertPage(QSharedPointer<ComicPage>(new ComicPage(layoutFilename, orientation)), index);
}

void ComicProject::removePage(QSharedPointer<ComicPage> page)
{
    m_private->pages.removeOne(page);
}

void ComicProject::removePage(int index)
{
    m_private->pages.removeAt(index);
}

QList< QSharedPointer<ComicPage> > ComicProject::pages() const
{
    return m_private->pages;
}

QString ComicProject::author() const
{
    return m_private->author;
}

QString ComicProject::title() const
{
    return m_private->title;
}

QString ComicProject::foldername() const
{
    return m_private->foldername;
}

void ComicProject::setAuthor(QString author)
{
    if (m_private->author == author)
        return;

    m_private->author = author;
    emit authorChanged(author);
}

void ComicProject::setTitle(QString title)
{
    if (m_private->title == title)
        return;

    m_private->title = title;
    emit titleChanged(title);
}

void ComicProject::setFoldername(QString foldername)
{
    if (m_private->foldername == foldername)
        return;

    m_private->foldername = foldername;
    emit foldernameChanged(foldername);
}

